from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.player_collection import PlayerCollection
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    search: Union[Unset, str] = UNSET,
    page: Union[Unset, float] = UNSET,
    countries: Union[Unset, str] = UNSET,
    with_metadata: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["search"] = search

    params["page"] = page

    params["countries"] = countries

    params["withMetadata"] = with_metadata

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/api/players",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, List["PlayerCollection"]]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = PlayerCollection.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = cast(Any, None)
        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, List["PlayerCollection"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    search: Union[Unset, str] = UNSET,
    page: Union[Unset, float] = UNSET,
    countries: Union[Unset, str] = UNSET,
    with_metadata: Union[Unset, bool] = UNSET,
) -> Response[Union[Any, List["PlayerCollection"]]]:
    """Get a list of players based on filters

    Args:
        search (Union[Unset, str]):
        page (Union[Unset, float]):
        countries (Union[Unset, str]):
        with_metadata (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['PlayerCollection']]]
    """

    kwargs = _get_kwargs(
        search=search,
        page=page,
        countries=countries,
        with_metadata=with_metadata,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    search: Union[Unset, str] = UNSET,
    page: Union[Unset, float] = UNSET,
    countries: Union[Unset, str] = UNSET,
    with_metadata: Union[Unset, bool] = UNSET,
) -> Optional[Union[Any, List["PlayerCollection"]]]:
    """Get a list of players based on filters

    Args:
        search (Union[Unset, str]):
        page (Union[Unset, float]):
        countries (Union[Unset, str]):
        with_metadata (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['PlayerCollection']]
    """

    return sync_detailed(
        client=client,
        search=search,
        page=page,
        countries=countries,
        with_metadata=with_metadata,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    search: Union[Unset, str] = UNSET,
    page: Union[Unset, float] = UNSET,
    countries: Union[Unset, str] = UNSET,
    with_metadata: Union[Unset, bool] = UNSET,
) -> Response[Union[Any, List["PlayerCollection"]]]:
    """Get a list of players based on filters

    Args:
        search (Union[Unset, str]):
        page (Union[Unset, float]):
        countries (Union[Unset, str]):
        with_metadata (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['PlayerCollection']]]
    """

    kwargs = _get_kwargs(
        search=search,
        page=page,
        countries=countries,
        with_metadata=with_metadata,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    search: Union[Unset, str] = UNSET,
    page: Union[Unset, float] = UNSET,
    countries: Union[Unset, str] = UNSET,
    with_metadata: Union[Unset, bool] = UNSET,
) -> Optional[Union[Any, List["PlayerCollection"]]]:
    """Get a list of players based on filters

    Args:
        search (Union[Unset, str]):
        page (Union[Unset, float]):
        countries (Union[Unset, str]):
        with_metadata (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['PlayerCollection']]
    """

    return (
        await asyncio_detailed(
            client=client,
            search=search,
            page=page,
            countries=countries,
            with_metadata=with_metadata,
        )
    ).parsed
